-------------------------------------------------------------------------------
-- DISCLAIMER OF LIABILITY
--
-- DLP Design is providing this design, code, or information "as-is" solely for 
-- use in developing programs and solutions for DLP Design products, with no 
-- obligation on the part of DLP Design to provide support. DLP Design expressly 
-- disclaims any warranty whatsoever with respect to the adequacy of the 
-- implementation, including but not limited to any warranties or representations 
-- that this implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular purpose.
--
-- DLP Design products are not intended for use in life support appliances, devices, 
-- or systems. Use in such applications is expressly prohibited.
--
-- Any modifications that are made to the Source Code are done at the user's sole 
-- risk and will be unsupported.
--
-- Copyright (c) 2006-2009 DLP Design, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part of this text at all times.
----------------------------------------------------------------------------------
-- Company: DLP Design, Inc., www.dlpdesign.com
-- 
-- Create Date:    10:36:40 12/27/2006 
-- Design Name:    usb_test
-- Module Name:    cmd_exec - Behavioral 
-- Project Name:   DLP_FPGA
-- Target Devices: XC3S250E-4TQ144C
-- Tool versions:  ISE 9.1.03i
-- Description:    Module interfaces to USB state machine to 
--			 execute pin and SRAM access instructions
--			 Sends bytes as appropriate based on commands
--                 
-- Dependencies:   Commmands 30,40,50&60 require cmd_exec module
--
-- Revision: 
-- 	0.01 	12/27/06	File Created
--    0.02  02/26/07 	Synced up with final version of USB state machine
--    0.03  02/28/07 	Modified RAM state machine to start accesses
--                      on the rising edge of w_ram, r_ram
--    1.00  07/31/07    Final Release.
--
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity cmd_exec is
    Port ( clk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           cmd_ready : in  STD_LOGIC;
           exec_done : out  STD_LOGIC;
           cmd_in : in  STD_LOGIC_VECTOR (7 downto 0);
           result_out : out  STD_LOGIC_VECTOR (7 downto 0);
			  -- SRAM signals
           r_ram : in  STD_LOGIC;
           w_ram : in  STD_LOGIC;
			  ram_address : in std_logic_vector (16 downto 0);
			  ram_data : in std_logic_vector (7 downto 0);
			  a : OUT std_logic_vector(16 downto 0);
			  din : IN std_logic_vector(7 downto 0);
			  dout : OUT std_logic_vector(7 downto 0);
			  dir : OUT std_logic;
			  oe : OUT std_logic;
			  we : OUT std_logic;
			  -- Pin Signals
			  pinget : in  STD_LOGIC;
           pinset : in  STD_LOGIC;
           pinclr : in  STD_LOGIC;
			  user_io_in : in std_logic_vector(28 downto 0);
			  user_io_out : out std_logic_vector(28 downto 0);
			  user_io_dir : out std_logic_vector(28 downto 0);
			  user_in : in std_logic_vector(10 downto 0));
end cmd_exec;

architecture Behavioral of cmd_exec is

   type state_type is (idle,ready,valid,pause1,pause2,pause3,done); -- 7 states
   signal state, next_state : state_type; 

	signal exec_done_i : std_logic;
	
	type rstate_type is (st1_w_idle, st2_w_addr, st3_w_data, st4_w_hold,
	                    st5_w_d_en, st6_w_d_a, st7_r_idle, st8_r_addr,
							  st9_r_oe, st10_r_hold, st11_r_data); 
   signal rstate, next_rstate : rstate_type; 

   signal oe_int : STD_LOGIC;
   signal we_int : STD_LOGIC;
   signal dir_int : STD_LOGIC;
	signal r_ram_d,w_ram_d : STD_LOGIC;

	
	signal iuser_io_out : std_logic_vector (28 downto 0);
	signal iuser_io_dir : std_logic_vector (28 downto 0);

begin

   SYNC_PROC: process (clk)
   begin
      if (clk'event and clk = '1') then
         if (reset = '1') then
            state 		<= idle;
				exec_done	<= '0';
			else
				state			<= next_state;
				exec_done	<= exec_done_i;
         end if;        
      end if;
   end process;

   --MOORE State Machine - Outputs based on state only
   OUTPUT_DECODE: process (state)
   begin
		case (state) is
			---------------------------------------------------------------------
			-- IDLE: Waiting for cmd_ready to go high
			when idle =>
            exec_done_i 	<= '0';
			---------------------------------------------------------------------
			-- READY: Read the state of the command inputs, determine next state
			when ready =>
            exec_done_i 	<= '0';
			---------------------------------------------------------------------
			-- valid: Command validated
			when valid =>
            exec_done_i 	<= '0';
			---------------------------------------------------------------------
			-- PAUSE1: Pause 1st clock cycle
			when pause1 =>
            exec_done_i 	<= '0';
			---------------------------------------------------------------------
			-- PAUSE2: Pause 2nd clock cycle
			when pause2 =>
            exec_done_i 	<= '0';
			---------------------------------------------------------------------
			-- PAUSE3: Pause 3rd clock cycle
			when pause3 =>
            exec_done_i 	<= '0';
			---------------------------------------------------------------------
			-- DONE: Command executed, assert done signal
			when done =>
            exec_done_i 	<= '1';
			---------------------------------------------------------------------
			-- OTHERS: 
			when others =>
            exec_done_i 	<= '0';
      end case;      
	end process;
 
   NEXT_STATE_DECODE: process (state,cmd_ready,pinget,pinclr,pinset,r_ram,w_ram)
   begin
      --declare default state for next_state to avoid latches
      next_state <= state;  --default is to stay in current state
		case (state) is
			---------------------------------------------------------------------
			-- IDLE: Waiting for ready_cmd to go high
			when idle =>
            if cmd_ready = '1' then -- command ready for execution
				   next_state <= ready;
				else
				   next_state <= idle;
				end if;
			---------------------------------------------------------------------
			-- READY: Determine command type
			when ready =>
            if (pinget='1' or pinclr='1' or pinset='1' or r_ram='1' 
				               or w_ram='1') then -- valid command
				   next_state <= valid;
				else
				   next_state <= idle;
				end if;
			---------------------------------------------------------------------
			-- VALID: Command is pin or ram access
			when valid =>
				next_state <= pause1;
			---------------------------------------------------------------------
			-- PAUSE1: Check command type
			when pause1 =>
            if (pinget='1' or pinclr='1' or pinset='1') then -- pin access
				   next_state <= done;
				else -- ram access
				   next_state <= pause2;
				end if;
			---------------------------------------------------------------------
			-- PAUSE2: Command is ram access
			when pause2 =>
				next_state <= pause3;
			---------------------------------------------------------------------
			-- PAUSE3: Command is ram access
			when pause3 =>
				next_state <= done;
			---------------------------------------------------------------------
			-- DONE: Command is ram access
			when done =>
				next_state <= idle;
			---------------------------------------------------------------------
			-- OTHERS
			when others =>
				next_state <= idle;
      end case;      
	end process;

   ACCESS_PROC: process (clk)
   begin
      if (clk'event and clk = '1') then
         if (reset = '1') then
            iuser_io_out 	<= (others => '0');
            iuser_io_dir 	<= (others => '1'); -- hi sets all IO to inputs
				result_out		<= X"00";
			else
				------------- PINCLR or PINSET command ---------------------
			   if (pinclr = '1' or pinset = '1') then
					case (cmd_in) is
						---------- value =  0, user_io(0) selected ---------
					   when X"00" => 	iuser_io_out(0) <= pinset;
											iuser_io_dir(0) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value =  1, user_io(1) selected ---------
					   when X"01" => 	iuser_io_out(1) <= pinset;
											iuser_io_dir(1) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value =  2, user_io(2) selected ---------
					   when X"02" => 	iuser_io_out(2) <= pinset;
											iuser_io_dir(2) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value =  3, user_io(3) selected ---------
					   when X"03" => 	iuser_io_out(3) <= pinset;
											iuser_io_dir(3) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value =  4, user_io(4) selected ---------
					   when X"04" => 	iuser_io_out(4) <= pinset;
											iuser_io_dir(4) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value =  5, user_io(5) selected ---------
					   when X"05" => 	iuser_io_out(5) <= pinset;
											iuser_io_dir(5) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value =  6, user_io(6) selected ---------
					   when X"06" => 	iuser_io_out(6) <= pinset;
											iuser_io_dir(6) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value =  7, user_io(7) selected ---------
					   when X"07" => 	iuser_io_out(7) <= pinset;
											iuser_io_dir(7) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value =  8, user_io(8) selected ---------
					   when X"08" => 	iuser_io_out(8) <= pinset;
											iuser_io_dir(8) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value =  9, user_io(9) selected ---------
					   when X"09" => 	iuser_io_out(9) <= pinset;
											iuser_io_dir(9) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 10, user_io(10) selected ---------
					   when X"0A" => 	iuser_io_out(10) <= pinset;
											iuser_io_dir(10) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 11, user_io(11) selected ---------
					   when X"0B" => 	iuser_io_out(11) <= pinset;
											iuser_io_dir(11) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 12, user_io(12) selected ---------
					   when X"0C" => 	iuser_io_out(12) <= pinset;
											iuser_io_dir(12) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 13, user_io(13) selected ---------
					   when X"0D" => 	iuser_io_out(13) <= pinset;
											iuser_io_dir(13) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 14, user_io(14) selected ---------
						--*** USER_IO(14) is actually an INPUT ONLY pin ***--
					   when X"0E" => 	iuser_io_out(14) <= pinset;
											iuser_io_dir(14) <= '0'; -- low for output
											result_out <= X"E5";
						---------- value = 15, user_io(15) selected ---------
						--*** USER_IO(15) is actually an INPUT ONLY pin ***--
					   when X"0F" => 	iuser_io_out(15) <= pinset;
											iuser_io_dir(15) <= '0'; -- low for output
											result_out <= X"E6"; -- return error 
						---------- value = 16, user_io(16) selected ---------
					   when X"10" => 	iuser_io_out(16) <= pinset;
											iuser_io_dir(16) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 17, user_io(17) selected ---------
					   when X"11" => 	iuser_io_out(17) <= pinset;
											iuser_io_dir(17) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 18, user_io(18) selected ---------
					   when X"12" => 	iuser_io_out(18) <= pinset;
											iuser_io_dir(18) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 19, user_io(19) selected ---------
					   when X"13" => 	iuser_io_out(19) <= pinset;
											iuser_io_dir(19) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 20, user_io(20) selected ---------
					   when X"14" => 	iuser_io_out(20) <= pinset;
											iuser_io_dir(20) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 21, user_io(21) selected ---------
					   when X"15" => 	iuser_io_out(21) <= pinset;
											iuser_io_dir(21) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 22, user_io(22) selected ---------
					   when X"16" => 	iuser_io_out(22) <= pinset;
											iuser_io_dir(22) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 23, user_io(23) selected ---------
					   when X"17" => 	iuser_io_out(23) <= pinset;
											iuser_io_dir(23) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 24, user_io(24) selected ---------
					   when X"18" => 	iuser_io_out(24) <= pinset;
											iuser_io_dir(24) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 25, user_io(25) selected ---------
					   when X"19" => 	iuser_io_out(25) <= pinset;
											iuser_io_dir(25) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 26, user_io(26) selected ---------
					   when X"1A" => 	iuser_io_out(26) <= pinset;
											iuser_io_dir(26) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 27, user_io(27) selected ---------
					   when X"1B" => 	iuser_io_out(27) <= pinset;
											iuser_io_dir(27) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = 28, user_io(28) selected ---------
					   when X"1C" => 	iuser_io_out(28) <= pinset;
											iuser_io_dir(28) <= '0'; -- low for output
											result_out <= cmd_in;
						---------- value = out of range ---------
						when others => iuser_io_out <= iuser_io_out;
											iuser_io_dir <= iuser_io_dir;
											if (pinclr = '1') then
												result_out <= X"E2"; -- pin clr specify error
											else
												result_out <= X"E3"; -- pin set specify error
											end if;
					end case;
				end if;
				------------- PINGET command ---------------------
			   if (pinget = '1') then
					case (cmd_in) is
						---------- value =  0, user_io(0) selected ---------
					   when X"00" => 	result_out <= "0000000" & user_io_in(0);
											iuser_io_dir(0) <= '1'; -- hi for input
						---------- value =  1, user_io(1) selected ---------
					   when X"01" => 	result_out <= "0000000" & user_io_in(1);
											iuser_io_dir(1) <= '1'; -- hi for input
						---------- value =  2, user_io(2) selected ---------
					   when X"02" => 	result_out <= "0000000" & user_io_in(2);
											iuser_io_dir(2) <= '1'; -- hi for input
						---------- value =  3, user_io(3) selected ---------
					   when X"03" => 	result_out <= "0000000" & user_io_in(3);
											iuser_io_dir(3) <= '1'; -- hi for input
						---------- value =  4, user_io(4) selected ---------
					   when X"04" => 	result_out <= "0000000" & user_io_in(4);
											iuser_io_dir(4) <= '1'; -- hi for input
						---------- value =  5, user_io(5) selected ---------
					   when X"05" => 	result_out <= "0000000" & user_io_in(5);
											iuser_io_dir(5) <= '1'; -- hi for input
						---------- value =  6, user_io(6) selected ---------
					   when X"06" => 	result_out <= "0000000" & user_io_in(6);
											iuser_io_dir(6) <= '1'; -- hi for input
						---------- value =  7, user_io(7) selected ---------
					   when X"07" => 	result_out <= "0000000" & user_io_in(7);
											iuser_io_dir(7) <= '1'; -- hi for input
						---------- value =  8, user_io(8) selected ---------
					   when X"08" => 	result_out <= "0000000" & user_io_in(8);
											iuser_io_dir(8) <= '1'; -- hi for input
						---------- value =  9, user_io(9) selected ---------
					   when X"09" => 	result_out <= "0000000" & user_io_in(9);
											iuser_io_dir(9) <= '1'; -- hi for input
						---------- value = 10, user_io(10) selected ---------
					   when X"0A" => 	result_out <= "0000000" & user_io_in(10);
											iuser_io_dir(10) <= '1'; -- hi for input
						---------- value = 11, user_io(11) selected ---------
					   when X"0B" => 	result_out <= "0000000" & user_io_in(11);
											iuser_io_dir(11) <= '1'; -- hi for input
						---------- value = 12, user_io(12) selected ---------
					   when X"0C" => 	result_out <= "0000000" & user_io_in(12);
											iuser_io_dir(12) <= '1'; -- hi for input
						---------- value = 13, user_io(13) selected ---------
					   when X"0D" => 	result_out <= "0000000" & user_io_in(13);
											iuser_io_dir(13) <= '1'; -- hi for input
						---------- value = 14, user_io(14) selected ---------
					   when X"0E" => 	result_out <= "0000000" & user_io_in(14);
											iuser_io_dir(14) <= '1'; -- hi for input
						---------- value = 15, user_io(15) selected ---------
					   when X"0F" => 	result_out <= "0000000" & user_io_in(15);
											iuser_io_dir(15) <= '1'; -- hi for input
						---------- value = 16, user_io(16) selected ---------
					   when X"10" => 	result_out <= "0000000" & user_io_in(16);
											iuser_io_dir(16) <= '1'; -- hi for input
						---------- value = 17, user_io(17) selected ---------
					   when X"11" => 	result_out <= "0000000" & user_io_in(17);
											iuser_io_dir(17) <= '1'; -- hi for input
						---------- value = 18, user_io(18) selected ---------
					   when X"12" => 	result_out <= "0000000" & user_io_in(18);
											iuser_io_dir(18) <= '1'; -- hi for input
						---------- value = 19, user_io(19) selected ---------
					   when X"13" => 	result_out <= "0000000" & user_io_in(19);
											iuser_io_dir(19) <= '1'; -- hi for input
						---------- value = 20, user_io(20) selected ---------
					   when X"14" => 	result_out <= "0000000" & user_io_in(20);
											iuser_io_dir(20) <= '1'; -- hi for input
						---------- value = 21, user_io(21) selected ---------
					   when X"15" => 	result_out <= "0000000" & user_io_in(21);
											iuser_io_dir(21) <= '1'; -- hi for input
						---------- value = 22, user_io(22) selected ---------
					   when X"16" => 	result_out <= "0000000" & user_io_in(22);
											iuser_io_dir(22) <= '1'; -- hi for input
						---------- value = 23, user_io(23) selected ---------
					   when X"17" => 	result_out <= "0000000" & user_io_in(23);
											iuser_io_dir(23) <= '1'; -- hi for input
						---------- value = 24, user_io(24) selected ---------
					   when X"18" => 	result_out <= "0000000" & user_io_in(24);
											iuser_io_dir(24) <= '1'; -- hi for input
						---------- value = 25, user_io(25) selected ---------
					   when X"19" => 	result_out <= "0000000" & user_io_in(25);
											iuser_io_dir(25) <= '1'; -- hi for input
						---------- value = 26, user_io(26) selected ---------
					   when X"1A" => 	result_out <= "0000000" & user_io_in(26);
											iuser_io_dir(26) <= '1'; -- hi for input
						---------- value = 27, user_io(27) selected ---------
					   when X"1B" => 	result_out <= "0000000" & user_io_in(27);
											iuser_io_dir(27) <= '1'; -- hi for input
						---------- value = 28, user_io(28) selected ---------
					   when X"1C" => 	result_out <= "0000000" & user_io_in(28);
										   iuser_io_dir(28) <= '1'; -- hi for input
						---------- value = 30, user_in(0) selected ---------
					   when X"1E" => 	result_out <= "0000000" & user_in(0);
						---------- value = 31, user_in(1) selected ---------
					   when X"1F" => 	result_out <= "0000000" & user_in(1);
						---------- value = 32, user_in(2) selected ---------
					   when X"20" => 	result_out <= "0000000" & user_in(2);
						---------- value = 33, user_in(3) selected ---------
					   when X"21" => 	result_out <= "0000000" & user_in(3);
						---------- value = 34, user_in(4) selected ---------
					   when X"22" => 	result_out <= "0000000" & user_in(4);
						---------- value = 35, user_in(5) selected ---------
					   when X"23" => 	result_out <= "0000000" & user_in(5);
						---------- value = 36, user_in(6) selected ---------
					   when X"24" => 	result_out <= "0000000" & user_in(6);
						---------- value = 37, user_in(7) selected ---------
					   when X"25" => 	result_out <= "0000000" & user_in(7);
						---------- value = 38, user_in(8) selected ---------
					   when X"26" => 	result_out <= "0000000" & user_in(8);
						---------- value = 39, user_in(9) selected ---------
					   when X"27" => 	result_out <= "0000000" & user_in(9);
						---------- value = 40, user_in(10) selected ---------
					   when X"28" => 	result_out <= "0000000" & user_in(10);
						---------- value = out of range ---------
						when others => iuser_io_dir <= iuser_io_dir;
											result_out <= X"E4";  -- pin get specify error
					end case;
				end if;
				------------- R_RAM command ---------------------
			   if (r_ram = '1') then
					result_out <= din;
				end if;
				------------- W_RAM command ---------------------
			   if (w_ram = '1') then
					result_out <= ram_data;
				end if;
				user_io_out <= iuser_io_out;
				user_io_dir <= iuser_io_dir;
         end if;        
      end if;
   end process;
	
	control_delay: process (clk)
	begin
		if (clk'event and clk = '1') then
			if (reset = '1') then
				r_ram_d <= '0';
				w_ram_d <= '0';
			else
				r_ram_d <= r_ram;
				w_ram_d <= w_ram;
         end if;        
      end if;
   end process;
			
	sram_sync: process (clk)
	begin
		if (clk'event and clk = '1') then
			if (reset = '1') then
				rstate <= st1_w_idle;
				oe 	<= '1';
				we 	<= '1';
				dir 	<= '1'; -- 1 = read, 0 = write (data out enabled)
			else
				rstate <= next_rstate;
				oe 	<= oe_int;
				we 	<= we_int;
				dir 	<= dir_int;
			end if;        
		end if;
	end process;
 
   --MOORE State Machine - Outputs based on state only
   sram_output_decode: process (rstate)
   begin      
		case (rstate) is
         when st1_w_idle =>
            oe_int 	<= '1';
            we_int 	<= '1';
				dir_int 	<= '1'; -- read (output disabled)
         when st2_w_addr =>
            oe_int 	<= '1';
            we_int 	<= '1';
				dir_int 	<= '0'; -- write (data out enabled)
         when st3_w_data =>
            oe_int 	<= '1';
            we_int 	<= '0';
				dir_int 	<= '0'; -- write (data out enabled)
         when st4_w_hold =>
            oe_int 	<= '1';
            we_int 	<= '0';
				dir_int 	<= '0'; -- write (data out enabled)
         when st5_w_d_en =>
            oe_int 	<= '1';
            we_int 	<= '1';
				dir_int 	<= '0'; -- write (data out enabled)
         when st6_w_d_a =>
            oe_int 	<= '1';
            we_int 	<= '1';
				dir_int 	<= '0'; -- write (data out enabled)
         when st7_r_idle =>
            oe_int 	<= '1';
            we_int 	<= '1';
				dir_int 	<= '1'; -- read (output disabled)
         when st8_r_addr =>
            oe_int 	<= '1';
            we_int 	<= '1';
				dir_int 	<= '1'; -- read (output disabled)
         when st9_r_oe =>
            oe_int 	<= '0';
            we_int 	<= '1';
				dir_int 	<= '1'; -- read (output disabled)
         when st10_r_hold =>
            oe_int 	<= '0';
            we_int 	<= '1';
				dir_int 	<= '1'; -- read (output disabled)
         when st11_r_data =>
            oe_int 	<= '0';
            we_int 	<= '1';
				dir_int 	<= '1'; -- read (output disabled)
         when others =>
            oe_int 	<= '1';
            we_int 	<= '1';
				dir_int 	<= '1'; -- read (output disabled)
      end case;      
   end process;
 
   sram_test_next_state: process (rstate, w_ram, r_ram)
   begin
      next_rstate <= rstate;  --default is to stay in current state
      case (rstate) is
         when st1_w_idle =>
				if (w_ram='1' and w_ram_d='0') then -- ram write access
					next_rstate <= st2_w_addr;
				elsif (r_ram = '1' and r_ram_d='0') then -- ram read access
					next_rstate <= st7_r_idle;
				else
					next_rstate <= st1_w_idle;
				end if;
         when st2_w_addr =>
            next_rstate <= st3_w_data;
         when st3_w_data =>
            next_rstate <= st4_w_hold;
         when st4_w_hold =>
            next_rstate <= st5_w_d_en;
         when st5_w_d_en =>
            next_rstate <= st6_w_d_a;
         when st6_w_d_a =>
				next_rstate <= st1_w_idle;
         when st7_r_idle =>
            next_rstate <= st8_r_addr;
         when st8_r_addr =>
            next_rstate <= st9_r_oe;
         when st9_r_oe =>
            next_rstate <= st10_r_hold;
         when st10_r_hold =>
            next_rstate <= st11_r_data;
         when st11_r_data =>
            next_rstate <= st1_w_idle;
         when others =>
            next_rstate <= st1_w_idle;
      end case;      
   end process;
	
	a <= ram_address;
	
   dout <= ram_data;

end Behavioral;

